import pygame
from pygame.locals import *
from pyogre import ogre 

class PyGameOGREApp:
    "Provides a base for an application using PyGame and PyOgre"
    def __init__(self, width=640, height=480, fullscreen=False):
        self._initPyGame()
        self._initPyOgre()
        self._createWindow(width, height, fullscreen)
        self._createViewport()
        self._loadResources("resources.cfg")
        self.init()

        self._createEntities()

        
    def init(self):
        pass
        
    def _initPyGame(self):
        "Starts up PyGame"
        pygame.init()
    def _initPyOgre(self):
        "Instantiates the PyOgre root and sceneManager objects and initialises"
        self.root = ogre.Root(ogre.getPluginPath())
        self.root.showConfigDialog()
        #renList = self.root.getAvailableRenderers()
        #print renList
        #for x in renList:
        #    print x.name
        #    if x.name.startswith( "D" ):
        #        self.root.renderSystem = x
        #        break
        self.root.initialise(False)
        self.sceneManager = self.root.getSceneManager(ogre.ST_GENERIC)
    def _createWindow(self, width, height, fullscreen):
        "Creates a PyGame window and sets PyOgre to render onto it"
        self.screen = pygame.display.set_mode((width,height))
        renderParameters = ogre._StringStringMap()
        renderParameters['externalWindowHandle'] = str(pygame.display.get_wm_info()['window'])
        self.renderWindow = self.root.createRenderWindow('PyOgre through PyGame', width, height, \
                                                         fullscreen, renderParameters)
        self.renderWindow.active = True
    def _createViewport(self):
        "Creates the user's viewport and camera"
        self.camera = self.sceneManager.createCamera("camera")
        self.camera.position = (0, 0, 500)
        self.camera.lookAt((0, 0, 0))
        self.camera.nearClipDistance = 5     
        self.viewport = self.renderWindow.addViewport(self.camera)
        self.viewport.backgroundColour = (0, 0, 0)
    def _loadResources(self, rcfile):
        "Loads the resource paths from specified resource config file"
        rc = ogre.ConfigFile()
        rc.loadFromFile(rcfile)
        for section, key, path in rc.values:
            ogre.ResourceGroupManager.getSingleton().addResourceLocation(path, key, section)
        ogre.ResourceGroupManager.getSingleton().initialiseAllResourceGroups()
    def createEntities(self):
        pass
    def _createEntities(self):
        self.createEntities()
        "For simple demo purposes this can be used to create entities and attach them to sceneNode's"
    def _createScene(self):
        "Prepare the scene. All logic should go here"
        self.update()
        pass
    def update(self):
        pass
    def _presentScene(self):
        "Render the scene and anything else needed done at the time of presentation"
        self.root.renderOneFrame()
    def _isPsycoEnabled(self):
        """Override this function and return True to turn on Psyco"""
        return True

    def _activatePsyco(self):        
        """Import Psyco if available"""
        try:
            import psyco
            psyco.full()
        except ImportError:
            pass

    def run(self):
        "Brings the application to life"
        if self._isPsycoEnabled():
            self._activatePsyco()
        while self._processEvents():
            self._createScene()
            self._presentScene()
        pygame.quit()
    def _processEvents(self):
        return self.processEvents()
    
    def processEvents(self):
        for event in pygame.event.get():
            if event.type is QUIT:
                return False
            elif event.type is KEYDOWN and event.key is K_ESCAPE:
                return False
        return True

